//
// This is an example macros for HBT analysis
//
#include <TROOT.h>
#include <TSystem.h>
#include <TChain.h>
#include <TFile.h>
#include <TString.h>
#include <iostream>

//
// To define cut monitor
// uncomments line below
//
// Cut monitors -- it is a bunch of histograms which shows
//                 what is passed and what is failed
//
//#define MONITORS

//
// To activate VPD_MB triggers uncommnect flag below
//
//#define VPD_MB

//
// In this analysis we construct correlation functions (CF's)
// for three (MULT_BINS) multiplicity bins.
// Why we did this? Because there is some physics behind that.
// see arXiv:nucl-ex/0505014 especially sections 4.1 and 4.4 for more info.
//
// Also we construct CF's not only for different multiplicity bins,
// but for different charge of particles in a pair (++ and --).
// For now (maybe I'm mistaken) we know that there is no difference between
// (++) and (--) CF's. But anyway we leave the opportunity to check that in this code.
//
// And another thing that is worse to mention. Actually, it is a very important thing.
// Besides charge and multiplicity we construct CF's for different transverse
// momentum of a pair (kT). There is a very important physics behind that.
// You can read more about it in the paper which I mentioned above in the section 4.4.
//
#define MULT_BINS 3
#define CHARGE_BINS 2

//
//  Event cuts
//
// Here and below you may see something like:
// const int mVal[2] = {-2., 2.};
// First element is lower limit for mVal
// Second element is upper limit for mVal
//
const int mPrimVertZbins = 6;                // number of z bins for analysis
const double mPrimVertZ[2] = {-30., 30.};    // redefine values of the vertex Z for each run [cm]
const double mPrimVertR = 2.;                // distance in the XY plane [cm]
const double mPrimVertVpdVz[2] = {-5., 5.};  // VPD Z - TPC Z difference
const double mPrimVertXShift = 0.;           // sometimes beam may be shifted in XY-plane
const double mPrimVertYShift = 0.;           //   this values defines shit of the beam along this axis
const bool mUseZdcCorrection = false;        // valid for AuAu collisions. True for 200GeV only!

//
// Track cuts
//
const double mChargedKaonMass = 0.493677;        // mass of the kaon [GeV]
const double mChargedPionMass = 0.13957018;      // mass of the pion [GeV]
const int mPosCharge = +1;                       // positive charge of the track
const int mNegCharge = -1;                       // negative charge of the track
const int mTrackType = 1;                        // 0 - global track, 1 - primary track
const int mPythiaKaonCode = 321;                 // pythia particle code. Not used in current analysis
const int mPythiaPionCode = 211;                 // pythia particle code
const int mTrackNHits[2] = {15, 50};             // number of track hits
const double mTrackNHitsFit[2] = {5, 50};        // number of track hits used in fit
const double mTrackDCA[2] = {0., 2.};            // distance of closest approach (DCA)
                                                 //   of the track to primary vertex [cm]
const double mTrackASplitVal = 0.51;             // nHitsFit/nHitsPossible. Remind us to tell you more
                                                 //  about it
const double mTrackMom[2] = {0.15, 1.55};        // momentum of the track [GeV/c]
const double mTrackTransvMom[2] = {0.05, 1.55};  // transverse momentum of the track [GeV/c]
const double mTrackEta[2] = { -1., 1.};          // pseudo-rapidity of the track

const short mDetectorSelection = 4;              // 0-TPC, 1-ToF, 2-TPC+ToF, 3-ToF||TPC
                                                 //   4-TPC&&TOF(P>Pthr)||TPC(P<Pthr)

const double mTrackTpcMom[2] = {0.15, 0.55};     // track momentum for TPC idnetification [GeV/c]
const double mnSigmaElectron[2] = {-1e9, 1e9};   // nSigma cut for electron hypothesis
const double mnSigmaPion[2] = {-1e9, 1e9};       // nSigma cut for pion hypothesis
const double mnSigmaKaon[2] = {-2., 2.};         // nSigma cut for kaon hypothesis. As you can see
                                                 //   there is different values. That is because
                                                 //   we want to identificate this particle with
                                                 //   2 sigmas. Easy peasy
const double mnSigmaProton[2] = {-1e9, 1e9};     // nSigma cut for proton hypothesis

const double mTrackTofMom[2] = {0.15, 1.45};     // track momentum for ToF identification [GeV/c]
const double mPionTofMsqr[2] = {-0.02, 0.062};   // mass square of a pion. Based on p+p data
const double mKaonTofMsqr[2] = {0.2, 0.32};      // mass square of a kaon. Based on p+p data

const double mTrackTnTMom[2] = {0.15, 1.55};      // momentum of track in case of TnT particle
                                                  //   identification (PID)
const double mTnTNSigmaElectron[2] = {-1e9, 1e9}; // nSigma for electrion in case of TnT PID
const double mTnTNSigmaPion[2] = {-1e9, 1e9};     // nSigma for pion in case of TnT PID
const double mTnTNSigmaKaon[2] = {-2., 2.};       // nSigma for kaon in case of TnT PID
const double mTnTNSigmaProton[2] = {-1e9, 1e9};   // nSigma for proton in case of TnT PID

const double mTTTMomThresh = 0.5; // TPC&&TOF (P>Pthr)||TPC(P<Pthr)

//
// Pair cuts
//
// Below we define merging (FMR) and splitting level (SL) cuts. You
// can read more about it there: arXiv:nucl-ex/0411036
//
const double mFmrCut[2] = {-1.1, 1.0};         // not more than 10% of merged rows
const double mSplitLevelCut[2] = {-0.5, 1.0};  // splitting level from paper. StHbtPair::quality()
const double mPairKt[2] = {-1e9., 1e9};        // pair transverse momentum cut
const double mPairQinv[2] = {-1e9., 1e9};      // pair relative momentum
const double mAveSep[2] = {5., 5000.};         // average separation of tracks within TPC

//
// Histograms values
//
const int mPairKtBins = 28;                 // bin width 50 MeV/c = kt_range/28
const double mPairKtVal[2] = {0.1, 1.5};    // range of pair transverse momentum
const int mQinvBins = 150;                  // 20 MeV/c^2 bin width
const double mQinvVal[2] = {0., 3.0};       // redefine for each analysis

const Char_t *defaultInFile = "test.list";                // input file name
const Char_t *defaultOutFile = "oExpKaon_pp200_1D.root";  // output file name


//
// This function takes couple of arguments, input file and output file names
//
void expKaonHBT_pp200_FemtoDst_1D(const Char_t *inFileList = defaultInFile,
                                  const Char_t *oFileName  = defaultOutFile)
{


  std::cout << "*****************************" << std::endl
            << "*    expKaonHBT_pp200_1D    *" << std::endl
            << "*          Start            *" << std::endl
            << "*****************************" << std::endl << std::endl;

  //
  // Load libraries
  //
  // You need to specift path to St-libraries by itself.
  // This St-libraries marked with wildcard comments.
  // You are free to use either absolute or relative path to that libraries.
  //
  gSystem->Load("libTable");
  gSystem->Load("libMinuit");
  gSystem->Load("libs/libStarClassLibrary.so");   // *
  gSystem->Load("libs/libSt_base.so");            // *
  gSystem->Load("libs/libStarRoot.so");           // *
  gSystem->Load("libs/libStChain.so");            // *
  gSystem->Load("libs/libStBichsel.so");          // *
  gSystem->Load("libs/libStEvent.so");            // *
  gSystem->Load("libs/libStStangeMuDstMaker.so"); // *
  gSystem->Load("libs/libStHbtMaker.so");         // *
  gSystem->Load("libs/libStFemto.so");            // *

  std::cout << "Libraries have been successfully loaded" << std::endl;

  //
  // Create chain. Chain is the class that calls other so-called maker classes.
  // It works like a chain where chain link is a maker class.
  //
  StChain* mChain = new StChain("StChain");
  mChain->SetDebug(0); // turn off debug verbose output

  //
  // Set StHbtMaker and StHbtManager
  //
  StHbtMaker *mHbtMaker = new StHbtMaker();
  mHbtMaker->SetDebug(0);
  StHbtManager *mHbtManager = mHbtMaker->HbtManager();

  //
  // Set StHbtReader. For more detailed info about classes and its methods see source code.
  //
  StHbtFemtoDstReader *femtoReader = new StHbtFemtoDstReader("", inFileList, "root", -1, false);
#ifdef VPD_MB
  femtoReader->AddTrigId(370011); // vpdmb-nobsmd
  femtoReader->AddTrigId(370001); // vpdmb
  femtoReader->AddTrigId(370022); // bbcmb
  femtoReader->AddTrigId(370501); // bht0-vpdmb
  femtoReader->AddTrigId(370511); // bht1-vpdmb
  femtoReader->AddTrigId(370531); // bht2
  femtoReader->AddTrigId(370542); // bht0-bbcmb-tof0
  femtoReader->AddTrigId(370546); // bht1-bbcmb-tof0
  femtoReader->AddTrigId(370522); // bht2-bbcmb
  femtoReader->AddTrigId(370601); // jp0
  femtoReader->AddTrigId(370611); // jp1
  femtoReader->AddTrigId(370621); // jp2
  femtoReader->AddTrigId(370641); // ajp
  femtoReader->AddTrigId(370301); // bbcmb-tof0
  femtoReader->AddTrigId(370361); // tofmult3-vpd
  femtoReader->AddTrigId(370341); // tofmult4

  //
  // Turn on only min-bias triggers
  //
  femtoReader->SetTrigger(370011, 1);
  femtoReader->SetTrigger(370001, 1);
#endif

  Long64_t mNEvents = femtoReader->GetNEvents(); // get number of events in the files loaded from a list
  mHbtManager->SetEventReader(femtoReader); // set event reader for HBT manager. There is a number of them.
                                            // Each reader can read specific file format.

  //
  // Set basic event cut. In the next code we will use this basic cuts for initialization
  // of our specific cuts used in actual analysis. We'll just use copy constructor
  //
  gregsEventCut *mBasicEventCut = new gregsEventCut();
  mBasicEventCut->SetCollisionType(0);     //0-pp, 1-AuAu
  mBasicEventCut->SetEventMult(0, 45);
  mBasicEventCut->SetVertZPos(mPrimVertZ[0], mPrimVertZ[1]);
  mBasicEventCut->SetVzVpdVzDiff(mPrimVertVpdVz[0],mPrimVertVpdVz[1]);
  mBasicEventCut->SetVertRPos(mPrimVertR);

  //
  // Set basic kaon single-particle cut
  //
  gregsSimpleTrackCut *mBasicTrackCut = new gregsSimpleTrackCut();
  mBasicTrackCut->SetCharge(mPosCharge);
  mBasicTrackCut->SetMass(mChargedKaonMass);
  mBasicTrackCut->SetType(mTrackType);
  mBasicTrackCut->SetNHits(mTrackNHits[0], mTrackNHits[1]);
  mBasicTrackCut->SetNHitsFit(mTrackNHitsFit[0], mTrackNHitsFit[1]);
  mBasicTrackCut->SetAntiSplit(mTrackASplitVal);
  mBasicTrackCut->SetEta(mTrackEta[0], mTrackEta[1]);
  mBasicTrackCut->SetDCA(mTrackDCA[0], mTrackDCA[1]);
  mBasicTrackCut->SetDCAGlobal(mTrackDCA[0], mTrackDCA[1]);
  mBasicTrackCut->SetP(mTrackMom[0], mTrackMom[1]);

  mBasicTrackCut->SetDetectorSelection(mDetectorSelection);

  mBasicTrackCut->SetTpcMomentum(mTrackTpcMom[0], mTrackTpcMom[1]);
  mBasicTrackCut->SetNSigmaElectron(mnSigmaElectron[0],mnSigmaElectron[1]);
  mBasicTrackCut->SetNSigmaPion(mnSigmaPion[0], mnSigmaPion[1]);
  mBasicTrackCut->SetNSigmaKaon(mnSigmaKaon[0], mnSigmaKaon[1]);
  mBasicTrackCut->SetNSigmaProton(mnSigmaProton[0], mnSigmaProton[1]);

  mBasicTrackCut->SetTofMassSqr(mKaonTofMsqr[0],mKaonTofMsqr[1]);
  mBasicTrackCut->SetTofMomentum(mTrackTofMom[0], mTrackTofMom[1]);

  mBasicTrackCut->SetTnTMomentum(mTrackTnTMom[0], mTrackTnTMom[1]);
  mBasicTrackCut->SetTnTNSigmaElectron(mTnTNSigmaElectron[0], mTnTNSigmaElectron[1]);
  mBasicTrackCut->SetTnTNSigmaPion(mTnTNSigmaPion[0], mTnTNSigmaPion[1]);
  mBasicTrackCut->SetTnTNSigmaKaon(mTnTNSigmaKaon[0], mTnTNSigmaKaon[1]);
  mBasicTrackCut->SetTnTNSigmaProton(mTnTNSigmaProton[0], mTnTNSigmaProton[1]);

  mBasicTrackCut->SetTTTMomentumThresh(mTTTMomThresh);

  //
  // Set basic kaon pair cut
  //
  gregsTrackPairCut *mBasicPairCut = new gregsTrackPairCut();
  mBasicPairCut->SetFracOfMergedRow(mFmrCut[0],mFmrCut[1]);
  mBasicPairCut->SetQuality(mSplitLevelCut[0],mSplitLevelCut[1]);
  mBasicPairCut->SetKt(mPairKt[0],mPairKt[1]);
  mBasicPairCut->SetQinv(mPairQinv[0],mPairQinv[1]);
  mBasicPairCut->SetAverageSeparation(mAveSep[0],mAveSep[1]);

  Int_t mLocalCharge, mMultBins, mMultLow, mMultHi, mEvents2Mix;
  //
  // The multiplicity analysis matrix is CHARGE_BINS x MULT_BINS, where
  // the first element corresponds to the particle charge: (pos, neg),
  // and the second one corresponds to the multiplicity intervals as listed below:
  // Mult: 0-200, 0-9, 10-200 (see below)
  //
  // StHbtAnalysis, cut, CF and monitor declarations
  //
  StHbtVertexMultAnalysis *mKaonMultAnalysis[CHARGE_BINS][MULT_BINS];

  //
  // Cuts
  //
  gregsEventCut       *mKaonEventCut[CHARGE_BINS][MULT_BINS];
  gregsSimpleTrackCut *mKaonTrackCut[CHARGE_BINS][MULT_BINS];
  gregsTrackPairCut   *mKaonPairCut[CHARGE_BINS][MULT_BINS];

  //
  // CorrFctns
  //
  QinvCorrFctnKt                *mKaonQinvMixKtCF[CHARGE_BINS];
  QinvCorrFctn                  *mKaonQinvMixMultCF[CHARGE_BINS][MULT_BINS];

#ifdef MONITORS
  //
  // Cut monitors
  //
  fxtEventCutMonitor *mFxtKaonEventCutMonitorPass[CHARGE_BINS][MULT_BINS];
  fxtTrackCutMonitor *mFxtKaonTrackCutMonitorPass[CHARGE_BINS][MULT_BINS];
  fxtPairCutMonitor  *mFxtKaonPairCutMonitorPass[CHARGE_BINS][MULT_BINS];
  fxtEventCutMonitor *mFxtKaonEventCutMonitorFail[CHARGE_BINS][MULT_BINS];
  fxtTrackCutMonitor *mFxtKaonTrackCutMonitorFail[CHARGE_BINS][MULT_BINS];
  fxtPairCutMonitor  *mFxtKaonPairCutMonitorFail[CHARGE_BINS][MULT_BINS];
#endif

  //
  // Create analyses
  //
  for(Int_t iMult = 0; iMult < MULT_BINS; iMult++)
  {
    for(Int_t iCharge = 0; iCharge < CHARGE_BINS; iCharge++)
    {
      switch(iMult) // set multiplicity range
      {
        case 1:  mMultLow = 0;  mMultHi = 9;    mMultBins = 1;  mEvents2Mix = 10 /* how much events will be used for event mixing */; break;
        case 2:  mMultLow = 10; mMultHi = 200;  mMultBins = 1;  mEvents2Mix = 10; break;
        default: mMultLow = 0;  mMultHi = 200;  mMultBins = 3;  mEvents2Mix = 10; break;
      };

      switch(iCharge)
      {
        case 0:   mLocalCharge = mPosCharge;    break;
        default:  mLocalCharge = mNegCharge;    break;
      };

      //
      // Create event cut
      //
      mKaonEventCut[iCharge][iMult] = new gregsEventCut(*mBasicEventCut);
      mKaonEventCut[iCharge][iMult]->SetEventMult(mMultLow, mMultHi);
#ifdef MONITORS
      mFxtKaonEventCutMonitorPass[iCharge][iMult] =
          new fxtEventCutMonitor("eventKaonPass", Form("_%d_%d", iCharge, iMult));
      mFxtKaonEventCutMonitorFail[iCharge][iMult] =
          new fxtEventCutMonitor("eventKaonFail", Form("_%d_%d", iCharge, iMult));
      mKaonEventCut[iCharge][iMult]->AddCutMonitor(mFxtKaonEventCutMonitorPass[iCharge][iMult],
                                                   mFxtKaonEventCutMonitorFail[iCharge][iMult]);
#endif

      //
      //  Like-sign kaon analysis
      //
      // create analysis
      mKaonMultAnalysis[iCharge][iMult] = new StHbtVertexMultAnalysis(mPrimVertZbins, // Z bins for mixing
                                                                      mPrimVertZ[0],  // lower Z limits
                                                                      mPrimVertZ[1],  // uppwer Z limits
                                                                      mMultBins,      // RefMult bins for mixing
                                                                      mMultLow,       // lower RefMult limits
                                                                      mMultHi);       // upper RefMult limits

      mKaonMultAnalysis[iCharge][iMult]->SetEventCut(mKaonEventCut[iCharge][iMult]);

      //
      // Create single-track cuts and add them to the analysis
      //
      mKaonTrackCut[iCharge][iMult] = new gregsSimpleTrackCut(*mBasicTrackCut);
      mKaonTrackCut[iCharge][iMult]->SetCharge(mLocalCharge);

#ifdef MONITORS
      mFxtKaonTrackCutMonitorPass[iCharge][iMult] = 
          new fxtTrackCutMonitor(Form("_trackKaonPass_%d_%d_", iCharge, iMult), mChargedKaonMass);
      mFxtKaonTrackCutMonitorFail[iCharge][iMult] = 
          new fxtTrackCutMonitor(Form("_trackKaonFail_%d_%d_", iCharge, iMult), mChargedKaonMass);
      mKaonTrackCut[iCharge][iMult]->AddCutMonitor(mFxtKaonTrackCutMonitorPass[iCharge][iMult],
                                                   mFxtKaonTrackCutMonitorFail[iCharge][iMult]);
#endif
      mKaonMultAnalysis[iCharge][iMult]->SetFirstParticleCut(mKaonTrackCut[iCharge][iMult]);
      mKaonMultAnalysis[iCharge][iMult]->SetSecondParticleCut(mKaonTrackCut[iCharge][iMult]);

      //
      // Create pair cut
      //
      mKaonPairCut[iCharge][iMult] = new gregsTrackPairCut(*mBasicPairCut);
#ifdef MONITORS
      mFxtKaonPairCutMonitorPass[iCharge][iMult] = 
          new fxtPairCutMonitor(Form("_pairKaonPass_%d_%d_", iCharge, iMult));
      mFxtKaonPairCutMonitorPass[iCharge][iMult]->SetParticleMass(mChargedKaonMass);
      mFxtKaonPairCutMonitorFail[iCharge][iMult] = 
          new fxtPairCutMonitor(Form("_pairKaonFail_%d_%d_", iCharge, iMult));
      mFxtKaonPairCutMonitorFail[iCharge][iMult]->SetParticleMass(mChargedKaonMass);
      mKaonPairCut[iCharge][iMult]->AddCutMonitor(mFxtKaonPairCutMonitorPass[iCharge][iMult],
                                                  mFxtKaonPairCutMonitorFail[iCharge][iMult]);
#endif
      mKaonMultAnalysis[iCharge][iMult]->SetPairCut(mKaonPairCut[iCharge][iMult]);

      //
      // Correlation function initialization
      //
      if (iMult == 0) 
      {
        mKaonQinvMixKtCF[iCharge] = 
            new QinvCorrFctnKt(Form("hKaonQinvMixKt_%d", iCharge),
                               mQinvBins, mQinvVal[0], mQinvVal[1],
                               mPairKtBins, mPairKtVal[0], mPairKtVal[1]);

        mKaonMultAnalysis[iCharge][iMult]->AddCorrFctn(mKaonQinvMixKtCF[iCharge]);
      }
      mKaonQinvMixMultCF[iCharge][iMult] =
          new QinvCorrFctn(Form("hKaonQinvMixMult_%d_%d", iCharge, iMult),
                           mQinvBins, mQinvVal[0], mQinvVal[1]);

      mKaonMultAnalysis[iCharge][iMult]->AddCorrFctn(mKaonQinvMixMultCF[iCharge][iMult]);

      mKaonMultAnalysis[iCharge][iMult]->SetNumEventsToMix(mEvents2Mix); // set number of events to mix
      mHbtManager->AddAnalysis(mKaonMultAnalysis[iCharge][iMult]); // add analysis to the manager
    }
  }

  mChain->Init(); // StChain initialization

  //
  // Start processing
  //
  Int_t iRet = 0;
  Int_t mNEventsProcessed = 0;
  Float_t mPercentCounter = 0.05;
  Float_t mProgress = 0.;

  unsigned int mSubCounter = 1;              // for ETA
  time_t mStartTime, mStopTime, mDiffTime;   //
  float mFrac;                               //
  mStartTime = time(0);                      //

  //
  // Start loop over events
  //
  std::cout << "Total Events: " << mNEvents << std::endl;
  for(Int_t iEvent=0; iEvent < mNEvents; iEvent++)
  {
    mProgress = (Float_t)iEvent/(Float_t)mNEvents; // calculate progress
    mNEventsProcessed++; // increase number of processed events

    //
    // Calculate ETA and output it
    //
    if(mProgress >= mPercentCounter)
    {
      mPercentCounter += 0.05;
      mStopTime = time(0);
      mDiffTime = difftime(mStopTime, mStartTime);
      mFrac = (float)mDiffTime*(float)(mNEvents - iEvent)/(float)iEvent;
      mSubCounter++;
      std::cout
          << Form("Processing progress: %4.2f%%. Time left: %.1f sec", mProgress*100., mFrac)
          << std::endl;
    }

    mChain->Clear();
    iRet = mChain->Make(iEvent); // if iRet == 0 then error occurs
    if (iRet != 0)
    {
      std::cout << "[ERROR] - iRet: " << iRet << std::endl;
      break;
    }
  }

  std::cout << "Calling Finish\n";
  mChain->Finish();

  std::cout << "Creating new file\n";
  TFile *oFile = new TFile(oFileName, "RECREATE"); // create output file

  //
  // Write hbt histograms to the file
  //
  std::cout << "Writing histograms to the output file...";

#ifdef MONITORS
  for(Int_t iCharge = 0; iCharge < CHARGE_BINS; iCharge++)
  {
    for(Int_t iMult = 0; iMult < MULT_BINS; iMult++)
    {
      mFxtKaonEventCutMonitorPass[iCharge][iMult]->VertexZ()->Write();
      mFxtKaonEventCutMonitorPass[iCharge][iMult]->VertexYvsVertexX()->Write();
      mFxtKaonEventCutMonitorPass[iCharge][iMult]->RefMult()->Write();
      mFxtKaonEventCutMonitorPass[iCharge][iMult]->VpdVzDiff()->Write();
      mFxtKaonEventCutMonitorFail[iCharge][iMult]->VertexZ()->Write();
      mFxtKaonEventCutMonitorFail[iCharge][iMult]->VertexYvsVertexX()->Write();
      mFxtKaonEventCutMonitorFail[iCharge][iMult]->RefMult()->Write();
      mFxtKaonEventCutMonitorFail[iCharge][iMult]->VpdVzDiff()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge][iMult]->DCAGlobal()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge][iMult]->Nhits()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge][iMult]->P()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge][iMult]->Pt()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge][iMult]->PtVsNsigmaPion()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge][iMult]->PtVsNsigmaKaon()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge][iMult]->PtVsNsigmaProton()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge][iMult]->PvsDedx()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge][iMult]->Rapidity()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge][iMult]->PseudoRapidity()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge][iMult]->PvsMassSqr()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge][iMult]->PvsInvBeta()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge][iMult]->DCAGlobal()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge][iMult]->Nhits()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge][iMult]->P()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge][iMult]->Pt()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge][iMult]->PtVsNsigmaPion()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge][iMult]->PtVsNsigmaKaon()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge][iMult]->PtVsNsigmaProton()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge][iMult]->PvsDedx()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge][iMult]->Rapidity()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge][iMult]->PseudoRapidity()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge][iMult]->PvsMassSqr()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge][iMult]->PvsInvBeta()->Write();

      mFxtKaonPairCutMonitorPass[iCharge][iMult]->Kt()->Write();
      mFxtKaonPairCutMonitorPass[iCharge][iMult]->Pt1Pt2Qinv()->Write();
      mFxtKaonPairCutMonitorPass[iCharge][iMult]->FMRvsQinv()->Write();
      mFxtKaonPairCutMonitorPass[iCharge][iMult]->SLvsQinv()->Write();
      mFxtKaonPairCutMonitorPass[iCharge][iMult]->AveSepVsQinv()->Write();
      mFxtKaonPairCutMonitorFail[iCharge][iMult]->Kt()->Write();
      mFxtKaonPairCutMonitorFail[iCharge][iMult]->Pt1Pt2Qinv()->Write();
      mFxtKaonPairCutMonitorFail[iCharge][iMult]->FMRvsQinv()->Write();
      mFxtKaonPairCutMonitorFail[iCharge][iMult]->SLvsQinv()->Write();
      mFxtKaonPairCutMonitorFail[iCharge][iMult]->AveSepVsQinv()->Write();
    }
  }

#endif

  for (Int_t iMult = 0; iMult < MULT_BINS; iMult++)
  {
    for (Int_t iCharge = 0; iCharge < CHARGE_BINS; iCharge++)
    {
      if (iMult == 0)
      {
        for (Int_t iKt = 0; iKt < mPairKtBins; iKt++)
        {
          mKaonQinvMixKtCF[iCharge]->Numerator(iKt)->Write();
          mKaonQinvMixKtCF[iCharge]->Denominator(iKt)->Write();
        }
      }
      mKaonQinvMixMultCF[iCharge][iMult]->Numerator()->Write();
      mKaonQinvMixMultCF[iCharge][iMult]->Denominator()->Write();
    }
  }
  
  std::cout << "\t[DONE]" << std::endl;

  //
  // Write the output file and close it
  //
  std::cout << "Saving the output file...";
  oFile->Close();
  std::cout << "\t[DONE]" << std::endl;

  std::cout << "********************************" << std::endl
            << "*       expKaonHBT_pp200_1D    *" << std::endl
            << "*              Finish          *" << std::endl
            << "********************************" << std::endl << std::endl;
}
