//
// This is an example macros for HBT analysis
//
#include <TROOT.h>
#include <TSystem.h>
#include <TChain.h>
#include <TFile.h>
#include <TString.h>
#include <iostream>

//
// To define cut monitor
// uncomments line below
//
// Cut monitors -- it is a bunch of histograms which shows
//                 what is passed and what is failed
//
//#define MONITORS

//
// To activate VPD_MB triggers uncommnect flag below
//
//#define VPD_MB


//
// In this analysis we construct invariant mass functions (Minv)
//

//
//  Event cuts
//
// Here and below you may see something like:
// const int mVal[2] = {-2., 2.};
// First element is lower limit for mVal
// Second element is upper limit for mVal
//
const int mPrimVertZbins = 6;                // number of z bins for analysis
const double mPrimVertZ[2] = {-30., 30.};    // redefine values of the vertex Z for each run [cm]
const double mPrimVertR = 2.;                // distance in the XY plane [cm]
const double mPrimVertVpdVz[2] = {-5., 5.};  // VPD Z - TPC Z difference
const double mPrimVertXShift = 0.;           // sometimes beam may be shifted in XY-plane
const double mPrimVertYShift = 0.;           //   this values defines shit of the beam along this axis
const bool mUseZdcCorrection = false;        // valid for AuAu collisions. True for 200GeV only!

//
// Track cuts
//
const double mChargedKaonMass = 0.493677;        // mass of the kaon [GeV]
const double mChargedPionMass = 0.13957018;      // mass of the pion [GeV]
const int mPosCharge = +1;                       // positive charge of the track
const int mNegCharge = -1;                       // negative charge of the track
const int mTrackType = 1;                        // 0 - global track, 1 - primary track
const int mPythiaKaonCode = 321;                 // pythia particle code. Not used in current analysis
const int mPythiaPionCode = 211;                 // pythia particle code
const int mTrackNHits[2] = {15, 50};             // number of track hits
const double mTrackNHitsFit[2] = {5, 50};        // number of track hits used in fit
const double mTrackDCA[2] = {0., 2.};            // distance of closest approach (DCA)
                                                 //   of the track to primary vertex [cm]
const double mTrackASplitVal = 0.51;             // nHitsFit/nHitsPossible. Remind us to tell you more
                                                 //  about it
const double mTrackMom[2] = {0.15, 1.55};        // momentum of the track [GeV/c]
const double mTrackTransvMom[2] = {0.05, 1.55};  // transverse momentum of the track [GeV/c]
const double mTrackEta[2] = { -1., 1.};          // pseudo-rapidity of the track

const short mDetectorSelection = 4;              // 0-TPC, 1-ToF, 2-TPC+ToF, 3-ToF||TPC
                                                 //   4-TPC&&TOF(P>Pthr)||TPC(P<Pthr)

const double mTrackTpcMom[2] = {0.15, 0.55};     // track momentum for TPC idnetification [GeV/c]
const double mnSigmaElectron[2] = {-1e9, 1e9};   // nSigma cut for electron hypothesis
const double mnSigmaPion[2] = {-1e9, 1e9};       // nSigma cut for pion hypothesis
const double mnSigmaKaon[2] = {-2., 2.};         // nSigma cut for kaon hypothesis. As you can see
                                                 //   there is different values. That is because
                                                 //   we want to identificate this particle with
                                                 //   2 sigmas. Easy peasy
const double mnSigmaProton[2] = {-1e9, 1e9};     // nSigma cut for proton hypothesis

const double mTrackTofMom[2] = {0.15, 1.45};     // track momentum for ToF identification [GeV/c]
const double mPionTofMsqr[2] = {-0.02, 0.062};   // mass square of a pion. Based on p+p data
const double mKaonTofMsqr[2] = {0.2, 0.32};      // mass square of a kaon. Based on p+p data

const double mTrackTnTMom[2] = {0.15, 1.55};      // momentum of track in case of TnT particle
                                                  //   identification (PID)
const double mTnTNSigmaElectron[2] = {-1e9, 1e9}; // nSigma for electrion in case of TnT PID
const double mTnTNSigmaPion[2] = {-1e9, 1e9};     // nSigma for pion in case of TnT PID
const double mTnTNSigmaKaon[2] = {-2., 2.};       // nSigma for kaon in case of TnT PID
const double mTnTNSigmaProton[2] = {-1e9, 1e9};   // nSigma for proton in case of TnT PID

const double mTTTMomThresh = 0.5; // TPC&&TOF (P>Pthr)||TPC(P<Pthr)

//
// Pair cuts
//
// Below we define merging (FMR) and splitting level (SL) cuts. You
// can read more about it there: arXiv:nucl-ex/0411036
//
const double mFmrCut[2] = {-1.1, 1.0};         // not more than 10% of merged rows
const double mSplitLevelCut[2] = {-0.5, 1.0};  // splitting level from paper. StHbtPair::quality()
const double mPairKt[2] = {-1e9., 1e9};        // pair transverse momentum cut
const double mPairQinv[2] = {-1e9., 1e9};      // pair relative momentum
const double mAveSep[2] = {5., 5000.};         // average separation of tracks within TPC

//
// Histograms values
//
const int mMinvBins = 500;             // 2 MeV bin width
const double mMinvVal[2] = {0.7, 1.7}; // redefine for each analysis

const Char_t *defaultInFile = "test.list";                // input file name
const Char_t *defaultOutFile = "oExpKaon_pp200_1D.root";  // output file name

//
// This function takes couple of arguments, input file and output file names
//
void expKaonMinv_pp200_FemtoDst_1D(const Char_t *inFileList = defaultInFile,
                                   const Char_t *oFileName  = defaultOutFile)
{


  std::cout << "*****************************" << std::endl
            << "*    expKaonMinv_pp200_1D   *" << std::endl
            << "*          Start            *" << std::endl
            << "*****************************" << std::endl << std::endl;

  //
  // Load libraries
  //
  // You need to specift path to St-libraries by itself.
  // This St-libraries marked with wildcard comments.
  // You are free to use either absolute or relative path to that libraries.
  //
  gSystem->Load("libTable");
  gSystem->Load("libMinuit");
  gSystem->Load("libs/libStarClassLibrary.so");   // *
  gSystem->Load("libs/libSt_base.so");            // *
  gSystem->Load("libs/libStarRoot.so");           // *
  gSystem->Load("libs/libStChain.so");            // *
  gSystem->Load("libs/libStBichsel.so");          // *
  gSystem->Load("libs/libStEvent.so");            // *
  gSystem->Load("libs/libStStangeMuDstMaker.so"); // *
  gSystem->Load("libs/libStHbtMaker.so");         // *
  gSystem->Load("libs/libStFemto.so");            // *

  std::cout << "Libraries have been successfully loaded" << std::endl;

  //
  // Create chain. Chain is the class that calls other so-called maker classes.
  // It works like a chain where chain link is a maker class.
  //
  StChain* mChain = new StChain("StChain");
  mChain->SetDebug(0); // turn off debug verbose output

  //
  // Set StHbtMaker and StHbtManager
  //
  StHbtMaker *mHbtMaker = new StHbtMaker();
  mHbtMaker->SetDebug(0);
  StHbtManager *mHbtManager = mHbtMaker->HbtManager();

  //
  // Set StHbtReader. For more detailed info about classes and its methods see source code.
  //
  StHbtFemtoDstReader *femtoReader = new StHbtFemtoDstReader("", inFileList, "root", -1, false);
#ifdef VPD_MB
  femtoReader->AddTrigId(370011); // vpdmb-nobsmd
  femtoReader->AddTrigId(370001); // vpdmb
  femtoReader->AddTrigId(370022); // bbcmb
  femtoReader->AddTrigId(370501); // bht0-vpdmb
  femtoReader->AddTrigId(370511); // bht1-vpdmb
  femtoReader->AddTrigId(370531); // bht2
  femtoReader->AddTrigId(370542); // bht0-bbcmb-tof0
  femtoReader->AddTrigId(370546); // bht1-bbcmb-tof0
  femtoReader->AddTrigId(370522); // bht2-bbcmb
  femtoReader->AddTrigId(370601); // jp0
  femtoReader->AddTrigId(370611); // jp1
  femtoReader->AddTrigId(370621); // jp2
  femtoReader->AddTrigId(370641); // ajp
  femtoReader->AddTrigId(370301); // bbcmb-tof0
  femtoReader->AddTrigId(370361); // tofmult3-vpd
  femtoReader->AddTrigId(370341); // tofmult4

  //
  // Turn on only min-bias triggers
  //
  femtoReader->SetTrigger(370011, 1);
  femtoReader->SetTrigger(370001, 1);
#endif

  Long64_t mNEvents = femtoReader->GetNEvents(); // get number of events in the files loaded from a list
  mHbtManager->SetEventReader(femtoReader); // set event reader for HBT manager. There is a number of them.
                                            // Each reader can read specific file format.

  //
  // Set basic event cut. In the next code we will use this basic cuts for initialization
  // of our specific cuts used in actual analysis. We'll just use copy constructor
  //
  gregsEventCut *mBasicEventCut = new gregsEventCut();
  mBasicEventCut->SetCollisionType(0);     //0-pp, 1-AuAu
  mBasicEventCut->SetEventMult(0, 45);
  mBasicEventCut->SetVertZPos(mPrimVertZ[0], mPrimVertZ[1]);
  mBasicEventCut->SetVzVpdVzDiff(mPrimVertVpdVz[0],mPrimVertVpdVz[1]);
  mBasicEventCut->SetVertRPos(mPrimVertR);

  //
  // Set basic kaon single-particle cut
  //
  gregsSimpleTrackCut *mBasicTrackCut = new gregsSimpleTrackCut();
  mBasicTrackCut->SetCharge(mPosCharge);
  mBasicTrackCut->SetMass(mChargedKaonMass);
  mBasicTrackCut->SetType(mTrackType);
  mBasicTrackCut->SetNHits(mTrackNHits[0], mTrackNHits[1]);
  mBasicTrackCut->SetNHitsFit(mTrackNHitsFit[0], mTrackNHitsFit[1]);
  mBasicTrackCut->SetAntiSplit(mTrackASplitVal);
  mBasicTrackCut->SetEta(mTrackEta[0], mTrackEta[1]);
  mBasicTrackCut->SetDCA(mTrackDCA[0], mTrackDCA[1]);
  mBasicTrackCut->SetDCAGlobal(mTrackDCA[0], mTrackDCA[1]);
  mBasicTrackCut->SetP(mTrackMom[0], mTrackMom[1]);

  mBasicTrackCut->SetDetectorSelection(mDetectorSelection);

  mBasicTrackCut->SetTpcMomentum(mTrackTpcMom[0], mTrackTpcMom[1]);
  mBasicTrackCut->SetNSigmaElectron(mnSigmaElectron[0],mnSigmaElectron[1]);
  mBasicTrackCut->SetNSigmaPion(mnSigmaPion[0], mnSigmaPion[1]);
  mBasicTrackCut->SetNSigmaKaon(mnSigmaKaon[0], mnSigmaKaon[1]);
  mBasicTrackCut->SetNSigmaProton(mnSigmaProton[0], mnSigmaProton[1]);

  mBasicTrackCut->SetTofMassSqr(mKaonTofMsqr[0],mKaonTofMsqr[1]);
  mBasicTrackCut->SetTofMomentum(mTrackTofMom[0], mTrackTofMom[1]);

  mBasicTrackCut->SetTnTMomentum(mTrackTnTMom[0], mTrackTnTMom[1]);
  mBasicTrackCut->SetTnTNSigmaElectron(mTnTNSigmaElectron[0], mTnTNSigmaElectron[1]);
  mBasicTrackCut->SetTnTNSigmaPion(mTnTNSigmaPion[0], mTnTNSigmaPion[1]);
  mBasicTrackCut->SetTnTNSigmaKaon(mTnTNSigmaKaon[0], mTnTNSigmaKaon[1]);
  mBasicTrackCut->SetTnTNSigmaProton(mTnTNSigmaProton[0], mTnTNSigmaProton[1]);

  mBasicTrackCut->SetTTTMomentumThresh(mTTTMomThresh);

  //
  // Set basic kaon pair cut
  //
  gregsTrackPairCut *mBasicPairCut = new gregsTrackPairCut();
  mBasicPairCut->SetFracOfMergedRow(mFmrCut[0],mFmrCut[1]);
  mBasicPairCut->SetQuality(mSplitLevelCut[0],mSplitLevelCut[1]);
  mBasicPairCut->SetKt(mPairKt[0],mPairKt[1]);
  mBasicPairCut->SetQinv(mPairQinv[0],mPairQinv[1]);
  mBasicPairCut->SetAverageSeparation(mAveSep[0],mAveSep[1]);

  Int_t mLocalCharge, mMultBins, mMultLow, mMultHi, mEvents2Mix;
  //
  // The multiplicity analysis matrix is CHARGE_BINS x MULT_BINS, where
  // the first element corresponds to the particle charge: (pos, neg),
  // and the second one corresponds to the multiplicity intervals as listed below:
  // Mult: 0-200, 0-9, 10-200 (see below)
  //
  // StHbtAnalysis, cut, CF and monitor declarations
  //
  StHbtVertexMultAnalysis *mKaonMultAnalysis;

  //
  // Cuts
  //
  gregsEventCut       *mKaonEventCut;
  gregsSimpleTrackCut *mKaonTrackCut[2]; // (+-) pair
  gregsTrackPairCut   *mKaonPairCut;

  //
  // Minv functions
  //
  MinvCorrFctn *mKaonMinv;

#ifdef MONITORS
  //
  // Cut monitors
  //
  fxtEventCutMonitor *mFxtKaonEventCutMonitorPass;
  fxtTrackCutMonitor *mFxtKaonTrackCutMonitorPass[2]; // (+-) pairs
  fxtPairCutMonitor  *mFxtKaonPairCutMonitorPass;
  fxtEventCutMonitor *mFxtKaonEventCutMonitorFail;
  fxtTrackCutMonitor *mFxtKaonTrackCutMonitorFail[2]; // (+-) pairs
  fxtPairCutMonitor  *mFxtKaonPairCutMonitorFail;
#endif

  //
  // Create analyses
  //
  mMultLow = 0;
  mMultHi = 200;
  mMultBins = 3;
  mEvents2Mix = 10;

  // create event cut
  mKaonEventCut = new gregsEventCut(*mBasicEventCut);
  mKaonEventCut->SetEventMult(mMultLow, mMultHi);
#ifdef MONITORS
  mFxtKaonEventCutMonitorPass = new fxtEventCutMonitor("eventKaonPass", "Minv");
  mFxtKaonEventCutMonitorFail = new fxtEventCutMonitor("eventKaonFail", "Minv");
  mKaonEventCut->AddCutMonitor(mFxtKaonEventCutMonitorPass,
                               mFxtKaonEventCutMonitorFail);
#endif

  // create analysis
  mKaonMultAnalysis = new StHbtVertexMultAnalysis(mPrimVertZbins, // Z bins for mixing
                                                  mPrimVertZ[0],  // lower Z limit
                                                  mPrimVertZ[1],  // upper Z limit
                                                  mMultBins,      // RefMult bins for mixing
                                                  mMultLow,       // lower RefMult limit
                                                  mMultHi);       // upper RefMult limit
  mKaonMultAnalysis->SetEventCut(mKaonEventCut);

  // create single-track cuts and add them to the analysis
  mKaonTrackCut[0] = new gregsSimpleTrackCut(*mBasicTrackCut); // positive track :)
  mKaonTrackCut[0]->SetCharge(mPosCharge);                     //
  mKaonTrackCut[1] = new gregsSimpleTrackCut(*mBasicTrackCut); // negative track :(
  mKaonTrackCut[1]->SetCharge(mNegCharge);                     //

#ifdef MONITORS
  // cut monitors for positive track
  mFxtKaonTrackCutMonitorPass[0] =
      new fxtTrackCutMonitor("_trackKaonPass_pos", mChargedKaonMass);
  mFxtKaonTrackCutMonitorFail[0] =
      new fxtTrackCutMonitor("_trackKaonFail_pos", mChargedKaonMass);
  mKaonTrackCut[0]->AddCutMonitor(mFxtKaonTrackCutMonitorPass[0],
                                  mFxtKaonTrackCutMonitorFail[0]);
  // cut monitors for negative track
  mFxtKaonTrackCutMonitorPass[1] =
      new fxtTrackCutMonitor("_trackKaonPass_neg", mChargedKaonMass);
  mFxtKaonTrackCutMonitorFail[1] =
      new fxtTrackCutMonitor("_trackKaonFail_neg", mChargedKaonMass);
  mKaonTrackCut[1]->AddCutMonitor(mFxtKaonTrackCutMonitorPass[1],
                                  mFxtKaonTrackCutMonitorFail[1]);
#endif
  mKaonMultAnalysis->SetFirstParticleCut(mKaonTrackCut[0]);
  mKaonMultAnalysis->SetSecondParticleCut(mKaonTrackCut[1]);

  // create pair cut
  mKaonPairCut = new gregsTrackPairCut(*mBasicPairCut);
#ifdef MONITORS
  mFxtKaonPairCutMonitorPass =
      new fxtPairCutMonitor("_pairKaonPass");
  mFxtKaonPairCutMonitorPass->SetParticleMass(mChargedKaonMass);
  mFxtKaonPairCutMonitorFail =
      new fxtPairCutMonitor("_pairKaonFail");
  mFxtKaonPairCutMonitorFail->SetParticleMass(mChargedKaonMass);
  mKaonPairCut->AddCutMonitor(mFxtKaonPairCutMonitorPass,
                              mFxtKaonPairCutMonitorFail);
#endif
  mKaonMultAnalysis->SetPairCut(mKaonPairCut);

  // Minv function initialization
  mKaonMinv = new MinvCorrFctn("hKaonMinv", mMinvBins, mMinvVal[0], mMinvVal[1]);

  mKaonMultAnalysis->AddCorrFctn(mKaonMinv);

  mKaonMultAnalysis->SetNumEventsToMix(mEvents2Mix); // set number of events to mix
  mHbtManager->AddAnalysis(mKaonMultAnalysis); // add analysis to the manager

  mChain->Init(); // StChain initialization

  //
  // Start processing
  //
  Int_t iRet = 0;
  Int_t mNEventsProcessed = 0;
  Float_t mPercentCounter = 0.05;
  Float_t mProgress = 0.;

  unsigned int mSubCounter = 1;              // for ETA
  time_t mStartTime, mStopTime, mDiffTime;   //
  float mFrac;                               //
  mStartTime = time(0);                      //

  //
  // Start loop over events
  //
  std::cout << "Total Events: " << mNEvents << std::endl;
  for(Int_t iEvent=0; iEvent < mNEvents; iEvent++)
  {
      mProgress = (Float_t)iEvent/(Float_t)mNEvents; // calculate progress
      mNEventsProcessed++; // increase number of processed events

      //
      // Calculate ETA and output it
      //
      if(mProgress >= mPercentCounter)
      {
          mPercentCounter += 0.05;
          mStopTime = time(0);
          mDiffTime = difftime(mStopTime, mStartTime);
          mFrac = (float)mDiffTime*(float)(mNEvents - iEvent)/(float)iEvent;
          mSubCounter++;
          std::cout
              << Form("Processing progress: %4.2f%%. Time left: %.1f sec", mProgress*100., mFrac)
              << std::endl;
      }

      mChain->Clear();
      iRet = mChain->Make(iEvent); // if iRet == 0 then error occurs
      if (iRet != 0)
      {
          std::cout << "[ERROR] - iRet: " << iRet << std::endl;
          break;
      }
  }

  std::cout << "Calling Finish\n";
  mChain->Finish();

  std::cout << "Creating new file\n";
  TFile *oFile = new TFile(oFileName, "RECREATE"); // create output file

  //
  // Write hbt histograms to the file
  //
  std::cout << "Writing histograms to the output file...";

#ifdef MONITORS
  // event cut monitors
  mFxtKaonEventCutMonitorPass->VertexZ()->Write();
  mFxtKaonEventCutMonitorPass->VertexYvsVertexX()->Write();
  mFxtKaonEventCutMonitorPass->RefMult()->Write();
  mFxtKaonEventCutMonitorPass->VpdVzDiff()->Write();
  mFxtKaonEventCutMonitorFail->VertexZ()->Write();
  mFxtKaonEventCutMonitorFail->VertexYvsVertexX()->Write();
  mFxtKaonEventCutMonitorFail->RefMult()->Write();
  mFxtKaonEventCutMonitorFail->VpdVzDiff()->Write();

  // track cut monitors
  for (int iCharge = 0; iCharge < 2; iCharge++)
  {
      mFxtKaonTrackCutMonitorPass[iCharge]->DCAGlobal()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge]->Nhits()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge]->P()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge]->Pt()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge]->PtVsNsigmaPion()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge]->PtVsNsigmaKaon()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge]->PtVsNsigmaProton()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge]->PvsDedx()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge]->Rapidity()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge]->PseudoRapidity()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge]->PvsMassSqr()->Write();
      mFxtKaonTrackCutMonitorPass[iCharge]->PvsInvBeta()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge]->DCAGlobal()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge]->Nhits()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge]->P()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge]->Pt()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge]->PtVsNsigmaPion()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge]->PtVsNsigmaKaon()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge]->PtVsNsigmaProton()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge]->PvsDedx()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge]->Rapidity()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge]->PseudoRapidity()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge]->PvsMassSqr()->Write();
      mFxtKaonTrackCutMonitorFail[iCharge]->PvsInvBeta()->Write();
  }

  // pair cut monitors
  mFxtKaonPairCutMonitorPass->Kt()->Write();
  mFxtKaonPairCutMonitorPass->Pt1Pt2Qinv()->Write();
  mFxtKaonPairCutMonitorPass->FMRvsQinv()->Write();
  mFxtKaonPairCutMonitorPass->SLvsQinv()->Write();
  mFxtKaonPairCutMonitorPass->AveSepVsQinv()->Write();
  mFxtKaonPairCutMonitorFail->Kt()->Write();
  mFxtKaonPairCutMonitorFail->Pt1Pt2Qinv()->Write();
  mFxtKaonPairCutMonitorFail->FMRvsQinv()->Write();
  mFxtKaonPairCutMonitorFail->SLvsQinv()->Write();
  mFxtKaonPairCutMonitorFail->AveSepVsQinv()->Write();
#endif

  mKaonMinv->Numerator()->Write();
  mKaonMinv->Denominator()->Write();
  mKaonMinv->Difference()->Write();

  std::cout << "\t[DONE]" << std::endl;

  //
  // Write the output file and close it
  //
  std::cout << "Saving the output file...";
  oFile->Close();
  std::cout << "\t[DONE]" << std::endl;

  std::cout
      << "********************************" << std::endl
      << "*       expKaonMinv_pp200_1D   *" << std::endl
      << "*              Finish          *" << std::endl
      << "********************************" << std::endl << std::endl;
}
